@@ -7,7 +7,7 @@ use diagnostics;
 use DB_File;
 use Text::CSV;
 
-my $path = "CSV/";
+my $path = 'CSV/';
 my $csv = Text::CSV->new();
 my $status;
 my @columns;
@@ -16,14 +16,10 @@ my @files = ('ECO','NIC','Opening');
 
 mkdir('db');
 foreach my $file (@files) {
-    if (-e "db/$file") {
-        print "$file exists ok\n";
-        next;
-    }
     print "Building $file ";
     tie %h, "DB_File", "db/$file", O_RDWR|O_CREAT, 0644, $DB_HASH
-        or die "Couldn't open '$file': $!\n";
-    open(IN,"$path$file.txt") or die "Couldn't open '$path$file.txt': $!\n";
+        or die "Couldn't tie '$file': $!\n";
+    open(IN,'<',"$path$file.txt") or die "Couldn't open '$path$file.txt': $!\n";
     while (<IN>) {
         $status = $csv->parse($_);
         @columns = $csv->fields();
